# sage_setup: distribution = sagemath-environment
r"""
Features for testing the presence of ``coxeter3``
"""

# *****************************************************************************
#       Copyright (C) 2016      Julian Rüth
#                     2018      Jeroen Demeyer
#                     2021-2024 Matthias Koeppe
#
#  Distributed under the terms of the GNU General Public License (GPL)
#  as published by the Free Software Foundation; either version 2 of
#  the License, or (at your option) any later version.
#                  https://www.gnu.org/licenses/
# *****************************************************************************

from . import PythonModule
from .join_feature import JoinFeature


class Coxeter3(JoinFeature):
    r"""
    A :class:`~sage.features.Feature` which describes whether the :mod:`sage.libs.coxeter3`
    module is available in this installation of Sage.

    EXAMPLES::

        sage: from sage.features.coxeter3 import Coxeter3
        sage: Coxeter3().require()  # optional - coxeter3
    """
    def __init__(self):
        r"""
        TESTS::

            sage: from sage.features.coxeter3 import Coxeter3
            sage: Coxeter3()
            Feature('coxeter3')
        """
        JoinFeature.__init__(self, "coxeter3",
                             [PythonModule("sage.libs.coxeter3.coxeter",
                                           spkg="sagemath_coxeter3")])


def all_features():
    return [Coxeter3()]



# sage_setup: distribution = sagemath-environment
r"""
Join features
"""

# ****************************************************************************
#       Copyright (C) 2021-2022 Matthias Koeppe
#                     2021-2022 Kwankyu Lee
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#                  https://www.gnu.org/licenses/
# ****************************************************************************

from . import Feature, FeatureTestResult


class JoinFeature(Feature):
    r"""
    Join of several :class:`~sage.features.Feature` instances.

    This creates a new feature as the union of the given features. Typically
    these are executables of an SPKG. For an example, see
    :class:`~sage.features.rubiks.Rubiks`.

    Furthermore, this can be the union of a single feature. This is used to map
    the given feature to a more convenient name to be used in ``optional`` tags
    of doctests. Thus you can equip a feature such as a
    :class:`~sage.features.PythonModule` with a tag name that differs from the
    systematic tag name. As an example for this use case, see
    :class:`~sage.features.meataxe.Meataxe`.

    EXAMPLES::

        sage: from sage.features import Executable
        sage: from sage.features.join_feature import JoinFeature
        sage: F = JoinFeature("shell-boolean",
        ....:                 (Executable('shell-true', 'true'),
        ....:                  Executable('shell-false', 'false')))
        sage: F.is_present()
        FeatureTestResult('shell-boolean', True)
        sage: F = JoinFeature("asdfghjkl",
        ....:                 (Executable('shell-true', 'true'),
        ....:                  Executable('xxyyyy', 'xxyyyy-does-not-exist')))
        sage: F.is_present()
        FeatureTestResult('xxyyyy', False)
    """

    def __init__(self, name, features, spkg=None, url=None, description=None, type=None,
                 **kwds):
        """
        TESTS:

        The empty join feature is present::

            sage: from sage.features.join_feature import JoinFeature
            sage: JoinFeature("empty", ()).is_present()
            FeatureTestResult('empty', True)
        """
        if spkg is None:
            spkgs = set(f.spkg for f in features if f.spkg)
            if len(spkgs) > 1:
                raise ValueError('given features have more than one spkg; provide spkg argument')
            elif len(spkgs) == 1:
                spkg = next(iter(spkgs))
        if url is None:
            urls = set(f.url for f in features if f.url)
            if len(urls) > 1:
                raise ValueError('given features have more than one url; provide url argument')
            elif len(urls) == 1:
                url = next(iter(urls))
        if type is None:
            if any(f._spkg_type() == 'experimental' for f in features):
                type = 'experimental'
            elif any(f._spkg_type() == 'optional' for f in features):
                type = 'optional'
            else:
                type = 'standard'

        super().__init__(name, spkg=spkg, url=url, description=description, type=type, **kwds)
        self._features = features

    def _is_present(self):
        r"""
        Test for the presence of the join feature.

        EXAMPLES::

            sage: from sage.features.latte import Latte
            sage: Latte()._is_present()  # optional - latte_int
            FeatureTestResult('latte_int', True)
        """
        for f in self._features:
            test = f._is_present()
            if not test:
                return test
        return FeatureTestResult(self, True)

    def hide(self):
        r"""
        Hide this feature and all its joined features.

        EXAMPLES::

            sage: from sage.features.sagemath import sage__groups
            sage: f = sage__groups()
            sage: f.hide()
            sage: f._features[0].is_present()
            FeatureTestResult('sage.groups.perm_gps.permgroup', False)

            sage: f.require()
            Traceback (most recent call last):
            ...
            FeatureNotPresentError: sage.groups is not available.
            Feature `sage.groups` is hidden.
            Use method `unhide` to make it available again.
        """
        for f in self._features:
            f.hide()
        super().hide()

    def unhide(self):
        r"""
        Revert what :meth:`hide` did.

        EXAMPLES::

            sage: from sage.features.sagemath import sage__groups
            sage: f = sage__groups()
            sage: f.hide()
            sage: f.is_present()
            FeatureTestResult('sage.groups', False)
            sage: f._features[0].is_present()
            FeatureTestResult('sage.groups.perm_gps.permgroup', False)

            sage: f.unhide()
            sage: f.is_present()    # optional sage.groups
            FeatureTestResult('sage.groups', True)
            sage: f._features[0].is_present() # optional sage.groups
            FeatureTestResult('sage.groups.perm_gps.permgroup', True)
        """
        for f in self._features:
            f.unhide()
        super().unhide()



# sage_setup: distribution = sagemath-environment
r"""
Feature for testing the presence of ``jupymake``, the Python interface to polymake
"""

# *****************************************************************************
#       Copyright (C) 2021 Matthias Koeppe
#
#  Distributed under the terms of the GNU General Public License (GPL)
#  as published by the Free Software Foundation; either version 2 of
#  the License, or (at your option) any later version.
#                  https://www.gnu.org/licenses/
# *****************************************************************************

from . import PythonModule
from .join_feature import JoinFeature


class JuPyMake(JoinFeature):
    r"""
    A :class:`~sage.features.Feature` describing the presence of the :ref:`JuPyMake <spkg_jupymake>`
    module, a Python interface to the :ref:`polymake <spkg_polymake>` library.

    EXAMPLES::

        sage: from sage.features.polymake import JuPyMake
        sage: JuPyMake().is_present()  # optional - jupymake
        FeatureTestResult('jupymake', True)
    """
    def __init__(self):
        r"""
        TESTS::

            sage: from sage.features.polymake import JuPyMake
            sage: isinstance(JuPyMake(), JuPyMake)
            True
        """
        JoinFeature.__init__(self, "jupymake",
                             [PythonModule("JuPyMake", spkg="jupymake")])


def all_features():
    return [JuPyMake()]



# Sage version information for Python scripts
# This file is auto-generated by the sage-update-version script, do not edit!
version = '10.4.rc2'
date = '2024-07-04'
banner = 'SageMath version 10.4.rc2, Release Date: 2024-07-04'



# sage_setup: distribution = sagemath-environment
r"""
Features for testing the presence of nauty executables
"""

# *****************************************************************************
#       Copyright (C) 2022 Matthias Koeppe
#
#  Distributed under the terms of the GNU General Public License (GPL)
#  as published by the Free Software Foundation; either version 2 of
#  the License, or (at your option) any later version.
#                  https://www.gnu.org/licenses/
# *****************************************************************************

from sage.env import SAGE_NAUTY_BINS_PREFIX

from . import Executable
from .join_feature import JoinFeature


class NautyExecutable(Executable):
    r"""
    A :class:`~sage.features.Feature` which checks for executables from the :ref:`nauty <spkg_nauty>` package.

    EXAMPLES::

        sage: from sage.features.nauty import NautyExecutable
        sage: NautyExecutable('converseg').is_present()                                 # needs nauty
        FeatureTestResult('nauty_converseg', True)
    """
    def __init__(self, name):
        r"""
        TESTS::

            sage: from sage.features.nauty import NautyExecutable
            sage: isinstance(NautyExecutable('geng'), NautyExecutable)
            True
        """
        Executable.__init__(self, name=f"nauty_{name}",
                            executable=f"{SAGE_NAUTY_BINS_PREFIX}{name}",
                            spkg="nauty",
                            type="standard")


class Nauty(JoinFeature):
    r"""
    A :class:`~sage.features.Feature` describing the presence of the executables
    which comes as a part of :ref:`nauty <spkg_nauty>`.

    EXAMPLES::

        sage: from sage.features.nauty import Nauty
        sage: Nauty().is_present()                                                      # needs nauty
        FeatureTestResult('nauty', True)
    """
    def __init__(self):
        r"""
        TESTS::

            sage: from sage.features.nauty import Nauty
            sage: isinstance(Nauty(), Nauty)
            True
        """
        JoinFeature.__init__(self, "nauty",
                             [NautyExecutable(name)
                              for name in ('directg', 'gentourng', 'geng', 'genbg', 'gentreeg', 'genktreeg')])


def all_features():
    return [Nauty()]



# sage_setup: distribution = sagemath-environment
r"""
Features for testing the presence of the SageMath interfaces to ``gap`` and of GAP packages
"""
# *****************************************************************************
#       Copyright (C) 2016 Julian Rüth
#                     2018 Jeroen Demeyer
#
#  Distributed under the terms of the GNU General Public License (GPL)
#  as published by the Free Software Foundation; either version 2 of
#  the License, or (at your option) any later version.
#                  https://www.gnu.org/licenses/
# *****************************************************************************

from . import Feature, FeatureTestResult, PythonModule
from .join_feature import JoinFeature
from .sagemath import sage__libs__gap

class GapPackage(Feature):
    r"""
    A :class:`~sage.features.Feature` describing the presence of a GAP package.

    A GAP package is "present" if it *can be* loaded, not if it *has
    been* loaded.

    .. SEEALSO::

        :class:`Feature sage.libs.gap <~sage.features.sagemath.sage__libs__gap>`

    EXAMPLES::

        sage: from sage.features.gap import GapPackage
        sage: GapPackage("grape", spkg="gap_packages")
        Feature('gap_package_grape')
    """
    def __init__(self, package, **kwds):
        r"""
        TESTS::

            sage: from sage.features.gap import GapPackage
            sage: isinstance(GapPackage("grape", spkg="gap_packages"), GapPackage)
            True
        """
        Feature.__init__(self, f"gap_package_{package}", **kwds)
        self.package = package

    def _is_present(self):
        r"""
        Return whether or not the GAP package is present.

        If the package is installed but not yet loaded, it is loaded
        first. This does *not* check that the package is functional.

        EXAMPLES::

            sage: from sage.features.gap import GapPackage
            sage: GapPackage("grape", spkg="gap_packages")._is_present()  # optional - gap_package_grape
            FeatureTestResult('gap_package_grape', True)
        """
        try:
            from sage.libs.gap.libgap import libgap
        except ImportError:
            return FeatureTestResult(self, False,
                                     reason="sage.libs.gap is not available")

        # This returns "true" even if the package is already loaded.
        command = 'LoadPackage("{package}")'.format(package=self.package)
        presence = libgap.eval(command)

        if presence:
            return FeatureTestResult(self, True,
                    reason="`{command}` evaluated to `{presence}` in GAP.".format(command=command, presence=presence))
        else:
            return FeatureTestResult(self, False,
                    reason="`{command}` evaluated to `{presence}` in GAP.".format(command=command, presence=presence))


def all_features():
    return [GapPackage("atlasrep", spkg="gap_packages"),
            GapPackage("design", spkg="gap_packages"),
            GapPackage("grape", spkg="gap_packages"),
            GapPackage("guava", spkg="gap_packages"),
            GapPackage("hap", spkg="gap_packages"),
            GapPackage("polycyclic", spkg="gap_packages"),
            GapPackage("qpa", spkg="gap_packages"),
            GapPackage("quagroup", spkg="gap_packages")]
